@@ -4,7 +4,8 @@ from feeluown.library import AbstractProvider, ProviderV2, ProviderFlags as PF,
     CommentModel, BriefCommentModel, BriefUserModel, UserModel, \
     NoUserLoggedIn, LyricModel, ModelNotFound
 from feeluown.media import Quality, Media
-from feeluown.models import ModelType, SearchType
+from feeluown.library import ModelType, SearchType
+from feeluown.utils.cache import cached_field
 from feeluown.utils.reader import create_reader, SequentialReader
 from .api import API
 
@@ -17,6 +18,7 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
     class meta:
         identifier = SOURCE
         name = '网易云音乐'
+        # TODO: remove
         flags = {
             ModelType.song: (PF.model_v2 | PF.similar | PF.multi_quality |
                              PF.get | PF.hot_comments | PF.web_url |
@@ -42,9 +44,10 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
         return '网易云音乐'
 
     def auth(self, user):
-        assert user.cookies is not None
+        cookies, exists = user.cache_get('cookies')
+        assert exists and cookies is not None
         self._user = user
-        self.api.load_cookies(user.cookies)
+        self.api.load_cookies(cookies)
 
     def has_current_user(self):
         return self._user is not None
@@ -63,6 +66,65 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
                          avatar_url=data['avatarImg'])
         return user
 
+    def current_user_fav_djradios(self):
+        return create_g(self.api.subscribed_djradio, NeteaseDjradioSchema, 'djRadios')
+
+    def current_user_fav_artists(self):
+        return create_g(self.api.user_favorite_artists, V2BriefArtistSchema)
+
+    def current_user_fav_albums(self):
+        return create_g(self.api.user_favorite_albums, V2BriefAlbumSchema)
+
+    def current_user_cloud_songs(self):
+        return create_cloud_songs_g(
+            self.api.cloud_songs,
+            self.api.cloud_songs_detail,
+            V2SongSchemaForV3,
+            NCloudSchema,
+            data_key='simpleSong'
+        )
+
+    def current_user_playlists(self):
+        user_id = str(self._user.identifier)
+        data_playlists = self.api.user_playlists(user_id)
+        playlists = []
+        fav_playlists = []
+        for pl in data_playlists:
+            if str(pl['userId']) == user_id:
+                playlists.append(pl)
+            else:
+                fav_playlists.append(pl)
+        return [_deserialize(e, V2PlaylistSchema) for e in playlists], \
+            [_deserialize(e, V2PlaylistSchema) for e in fav_playlists]
+
+    def current_user_get_radio_songs(self):
+        songs_data = self.api.get_radio_music()
+        if songs_data is None:
+            logger.error('data should not be None')
+            return None
+        return [_deserialize(song_data, V2SongSchema)
+                for song_data in songs_data]
+
+    @cached_field()
+    def current_user_rec_playlists_p(self):
+        playlists_data = self.api.get_recommend_playlists()
+        rec_playlists = []
+        for playlist_data in playlists_data:
+            # FIXME: GUI模式下无法显示歌单描述
+            playlist_data['coverImgUrl'] = playlist_data['picUrl']
+            playlist_data['description'] = None
+            playlist = _deserialize(playlist_data, V2PlaylistSchema)
+            rec_playlists.append(playlist)
+        return rec_playlists
+
+    # 根据过去经验，每日推荐歌曲在每天早上 6:00 刷新，
+    # ttl 设置为 60s 是为了能够比较即时的获取今天推荐。
+    @cached_field(ttl=60)
+    def current_user_rec_songs_p(self):
+        songs_data = self.api.get_recommend_songs()
+        return [_deserialize(song_data, V2SongSchemaForV3)
+                for song_data in songs_data]
+
     def song_get(self, identifier):
         data = self.api.song_detail(int(identifier))
         return _deserialize(data, V2SongSchema)
@@ -73,12 +135,11 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
 
     def song_get_lyric(self, song):
         data = self.api.get_lyric_by_songid(song.identifier)
-        lrc = data.get('lrc', {})
-        lyric = lrc.get('lyric', '')
         return LyricModel(
             source=SOURCE,
             identifier=self.identifier,
-            content=lyric,
+            content=data.get('lrc', {}).get('lyric', ''),
+            trans_content=data.get('tlyric', {}).get('lyric', ''),
         )
 
     def song_get_mv(self, song):
@@ -245,6 +306,10 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
         album = _deserialize(album_data, V2AlbumSchema)
         return album
 
+    def album_create_songs_rd(self, album):
+        album_with_songs = self.album_get(album.identifier)
+        return create_reader(album_with_songs.songs)
+
     def artist_get(self, identifier):
         artist_data = self.api.artist_infos(identifier)
         artist = artist_data['artist']
@@ -289,7 +354,7 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
                         # the songs field will always be an empty list,
                         # we set it to None
                         album['songs'] = None
-                        yield _deserialize(album, V2BriefAlbumSchema)
+                        yield _deserialize(album, V2AlbumSchema)
                         cur += 1
                     if data['more']:
                         data = self.api.artist_albums(artist.identifier, offset=cur)
@@ -303,7 +368,25 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
         playlist = _deserialize(data, V2PlaylistSchema)
         return playlist
 
+    def djradio_create_songs_rd(self, djradio_id):
+        data = self.api.djradio_list(djradio_id, limit=1, offset=0)
+        count = data.get('count', 0)
+
+        def g():
+            offset = 0
+            per = 50  # speed up first request
+            while offset < count:
+                tracks_data = self.api.djradio_list(
+                    djradio_id, limit=per, offset=offset)
+                for track_data in tracks_data.get('programs', []):
+                    yield _deserialize(track_data, NDjradioSchema)
+                offset += per
+        return create_reader(g())
+
     def playlist_create_songs_rd(self, playlist):
+        if playlist.identifier.startswith(DjradioPrefix):
+            return self.djradio_create_songs_rd(playlist.identifier[len(DjradioPrefix):])
+
         data = self.api.playlist_detail_v3(playlist.identifier, limit=0)
         track_ids = data['trackIds']  # [{'id': 1, 'v': 1}, ...]
         count = len(track_ids)
@@ -359,22 +442,27 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
             SearchType.pl: 1000,
         }
         data = provider.api.search(keyword, stype=type_type_map[type_])
+        data['q'] = keyword
         result = _deserialize(data, NeteaseSearchSchema)
-        result.q = keyword
         return result
 
 
 provider = NeteaseProvider()
 
 
-from .models import _deserialize  # noqa
+from .models import _deserialize, create_g, create_cloud_songs_g  # noqa
 from .schemas import (  # noqa
     V2SongSchema,
     V2SongSchemaForV3,
     V2MvSchema,
     V2AlbumSchema,
     V2ArtistSchema,
+    V2BriefArtistSchema,
     V2BriefAlbumSchema,
     V2PlaylistSchema,
+    NeteaseDjradioSchema,
     NeteaseSearchSchema,
+    NDjradioSchema,
+    NCloudSchema,
+    DjradioPrefix,
 )
